@@ -33,7 +33,7 @@ AthenaConnection <-
     ...){
     
     tryCatch(
-      ptr <- boto$Session(aws_access_key_id = aws_access_key_id,
+      boto3 <- boto$Session(aws_access_key_id = aws_access_key_id,
                           aws_secret_access_key = aws_secret_access_key,
                           aws_session_token = aws_session_token,
                           region_name = region_name,
@@ -43,12 +43,15 @@ AthenaConnection <-
       error = function(e) py_error(e))
     
     # stop connection if region_name is not set in backend or hardcoded
-    if(is.null(ptr$region_name)) stop("AWS `region_name` is required to be set. Please set `region` in .config file, ",
+    if(is.null(boto3$region_name)) stop("AWS `region_name` is required to be set. Please set `region` in .config file, ",
                                       "`AWS_REGION` in environment variables or `region_name` hard coded in `dbConnect()`.", call. = FALSE)
     
+    ptr_ll <- list(Athena = boto3$client("athena"),
+                   S3 = boto3$client("s3"),
+                   glue = boto3$client("glue"))
+    
     if(is.null(s3_staging_dir) && !is.null(work_group)){
-      Athena <- ptr$client("athena")
-      tryCatch(s3_staging_dir <- Athena$get_work_group(WorkGroup = work_group)$WorkGroup$Configuration$ResultConfiguration$OutputLocation,
+      tryCatch(s3_staging_dir <- ptr_ll$Athena$get_work_group(WorkGroup = work_group)$WorkGroup$Configuration$ResultConfiguration$OutputLocation,
                error = function(e) py_error(e))
     }
     
@@ -60,9 +63,10 @@ AthenaConnection <-
     info <- list(profile_name = profile_name, s3_staging = s3_staging_dir,
                  dbms.name = schema_name, work_group = work_group %||% "primary",
                  poll_interval = poll_interval, encryption_option = encryption_option,
-                 kms_key = kms_key, expiration = aws_expiration, keyboard_interrupt = keyboard_interrupt)
+                 kms_key = kms_key, expiration = aws_expiration, keyboard_interrupt = keyboard_interrupt,
+                 region_name = boto3$region_name)
     
-    res <- new("AthenaConnection",  ptr = ptr, info = info, quote = "`")
+    res <- new("AthenaConnection",  ptr = list2env(ptr_ll, parent = emptyenv()), info = list2env(info, parent = emptyenv()), quote = "`")
   }
 
 #' @rdname AthenaConnection
@@ -72,8 +76,8 @@ setClass(
   "AthenaConnection",
   contains = "DBIConnection",
   slots = list(
-    ptr = "ANY",
-    info = "list",
+    ptr = "environment",
+    info = "environment",
     quote = "character" )
 )
 
@@ -119,7 +123,7 @@ setMethod(
       warning("Connection already closed.", call. = FALSE)
     } else {
       on_connection_closed(conn)
-      eval.parent(substitute(conn@ptr <- NULL))}
+      rm(list = ls(all.names = TRUE, envir = conn@ptr), envir = conn@ptr)}
     invisible(NULL)
   })
 
@@ -206,10 +210,11 @@ setMethod(
   "dbSendQuery", c("AthenaConnection", "character"),
   function(conn,
            statement = NULL, ...){
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    con_error_msg(conn, msg = "Connection already closed.")
     s3_staging_dir <- conn@info$s3_staging
-    res <- AthenaResult(conn =conn, statement= statement, s3_staging_dir = s3_staging_dir)
-    res
+    res <- AthenaResult(
+      conn=conn, statement= statement, s3_staging_dir = s3_staging_dir)
+    return(res)
   }
 )
 
@@ -219,10 +224,11 @@ setMethod(
   "dbSendStatement", c("AthenaConnection", "character"),
   function(conn,
            statement = NULL, ...){
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    con_error_msg(conn, msg = "Connection already closed.")
     s3_staging_dir <- conn@info$s3_staging
-    res <- AthenaResult(conn =conn, statement= statement, s3_staging_dir = s3_staging_dir)
-    res
+    res <- AthenaResult(
+      conn =conn, statement= statement, s3_staging_dir = s3_staging_dir)
+    return(res)
   }
 )
 
@@ -232,20 +238,21 @@ setMethod(
   "dbExecute", c("AthenaConnection", "character"),
   function(conn,
            statement = NULL, ...){
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    con_error_msg(conn, msg = "Connection already closed.")
     s3_staging_dir <- conn@info$s3_staging
-    res <- AthenaResult(conn =conn, statement= statement, s3_staging_dir = s3_staging_dir)
-    poll_result <- poll(res)
+    res <- AthenaResult(
+      conn=conn, statement= statement, s3_staging_dir = s3_staging_dir)
+    poll(res)
     
     # if query failed stop
-    if(poll_result$QueryExecution$Status$State == "FAILED") {
-      stop(poll_result$QueryExecution$Status$StateChangeReason, call. = FALSE)
-    }
+    if(res@info$Status == "FAILED")
+      stop(res@info$StateChangeReason, call. = FALSE)
     
     # cache query metadata if caching is enabled
-    if (athena_option_env$cache_size > 0) cache_query(poll_result)
+    if (athena_option_env$cache_size > 0)
+      cache_query(res)
     
-    res
+    return(res)
   }
 )
 
@@ -361,16 +368,19 @@ NULL
 setMethod(
   "dbListTables", "AthenaConnection",
   function(conn, schema = NULL,...){
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
-    glue <- conn@ptr$client("glue")
-    
-    if(is.null(schema)){
-      retry_api_call(schema <- sapply(glue$get_databases()$DatabaseList,function(x) x$Name))}
-    tryCatch(output <- lapply(schema, function (x) tryCatch(glue$get_tables(DatabaseName = x)$TableList, error = function(cond) NULL)),
-             error = function(e) py_error(e))
-    unlist(lapply(output, function(x) sapply(x, function(y) y$Name)))
-  }
-)
+    con_error_msg(conn, msg = "Connection already closed.")
+    glue <- conn@ptr$glue
+    if(is.null(schema))
+      schema <- get_databases(glue)
+    tryCatch({output <- lapply(schema, function(i) get_table_list(glue = glue, schema = i))},
+             error = function(cond) NULL)
+    return(
+      vapply(
+        unlist(output, recursive = FALSE),
+        function(y) y$Name,
+        FUN.VALUE = character(1))
+    )
+})
 
 #' List Athena Schema, Tables and Table Types
 #'
@@ -407,20 +417,20 @@ setGeneric("dbGetTables", function(conn, ...) standardGeneric("dbGetTables"))
 
 #' @rdname dbGetTables
 #' @export
-setMethod("dbGetTables", "AthenaConnection",
+setMethod(
+  "dbGetTables", "AthenaConnection",
   function(conn, schema = NULL, ...){
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
-    glue <- conn@ptr$client("glue")
-  if(is.null(schema)){
-    retry_api_call(schema <- sapply(glue$get_databases()$DatabaseList,function(x) x$Name))}
-  tryCatch(output <- lapply(schema, function (x) tryCatch(glue$get_tables(DatabaseName = x)$TableList, error = function(cond) NULL)),
-           error = function(e) py_error(e))
-  rbindlist(lapply(output, function(x) rbindlist(lapply(x, function(y) data.frame(Schema = y$DatabaseName,
-                                                                                  TableName=y$Name,
-                                                                                  TableType = y$TableType)))))
+    con_error_msg(conn, msg = "Connection already closed.")
+    glue <- conn@ptr$glue
+    if(is.null(schema))
+      schema <- get_databases(glue)
+    tryCatch({output <- lapply(schema, function(i) get_table_list(glue = glue, schema = i))},
+             error = function(cond) NULL)
+    output <- rbindlist(unlist(output, recursive = FALSE), use.names = TRUE)
+    setnames(output, new = c("Schema", "TableName", "TableType"))
+    return(output)
 })
 
-
 #' List Field names of Athena table
 #'
 #' @name dbListFields
@@ -455,27 +465,20 @@ NULL
 
 #' @rdname dbListFields
 #' @export
-setMethod("dbListFields", c("AthenaConnection", "character") ,
-          function(conn, name, ...) {
-            if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
-            
-            if (grepl("\\.", name)) {
-              dbms.name <- gsub("\\..*", "" , name)
-              Table <- gsub(".*\\.", "" , name)
-            } else {
-              dbms.name <- conn@info$dbms.name
-              Table <- name}
-            
-            glue <- conn@ptr$client("glue")
-            tryCatch(
-              output <- glue$get_table(DatabaseName = dbms.name,
-                                       Name = Table)$Table,
-              error = function(e) py_error(e))
-            col_names = vapply(output$StorageDescriptor$Columns, function(y) y$Name, FUN.VALUE = character(1))
-            partitions = vapply(output$PartitionKeys,function(y) y$Name, FUN.VALUE = character(1))
-            
-            c(col_names, partitions)
-          })
+setMethod(
+  "dbListFields", c("AthenaConnection", "character") ,
+  function(conn, name, ...) {
+    con_error_msg(conn, msg = "Connection already closed.")
+    ll <- db_detect(conn, name)
+    retry_api_call(
+      output <- conn@ptr$glue$get_table(
+        DatabaseName = ll[["dbms.name"]],
+        Name = ll[["table"]])$Table)
+    
+    col_names = vapply(output$StorageDescriptor$Columns, function(y) y$Name, FUN.VALUE = character(1))
+    partitions = vapply(output$PartitionKeys,function(y) y$Name, FUN.VALUE = character(1))
+    c(col_names, partitions)
+})
 
 #' Does Athena table exist?
 #' 
@@ -514,18 +517,11 @@ NULL
 setMethod(
   "dbExistsTable", c("AthenaConnection", "character"),
   function(conn, name, ...) {
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
-    
-    if(grepl("\\.", name)){
-      dbms.name <- gsub("\\..*", "" , name)
-      Table <- gsub(".*\\.", "" , name)
-    } else {dbms.name <- conn@info$dbms.name
-    Table <- tolower(name)}
-    
-    glue <- conn@ptr$client("glue")
+    con_error_msg(conn, msg = "Connection already closed.")
+    ll <- db_detect(conn, name)
 
     for (i in seq_len(athena_option_env$retry)) {
-      resp <- tryCatch(glue$get_table(DatabaseName = dbms.name, Name = Table), 
+      resp <- tryCatch(conn@ptr$glue$get_table(DatabaseName = ll[["dbms.name"]], Name = ll[["table"]]), 
                        error = function(e) retry_error(e))
       
       # exponential step back if error and not expected error
@@ -586,54 +582,58 @@ NULL
 setMethod(
   "dbRemoveTable", c("AthenaConnection", "character"),
   function(conn, name, delete_data = TRUE, confirm = FALSE, ...) {
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    con_error_msg(conn, msg = "Connection already closed.")
     stopifnot(is.logical(delete_data),
               is.logical(confirm))
+    ll <- db_detect(conn, name)
     
-    if (grepl("\\.", name)) {
-      dbms.name <- gsub("\\..*", "" , name)
-      Table <- gsub(".*\\.", "" , name)
-    } else {
-      dbms.name <- conn@info$dbms.name
-      Table <- name}
-    
-    glue <- conn@ptr$client("glue")
-    s3 <- conn@ptr$resource("s3")
-    
-    tryCatch(TableType <- glue$get_table(DatabaseName = dbms.name, Name = Table)$Table$TableType,
+    tryCatch(TableType <- conn@ptr$glue$get_table(DatabaseName = ll[["dbms.name"]], Name = ll[["table"]])[["Table"]][["TableType"]],
              error = function(e) py_error(e))
     
     if (delete_data && TableType == "EXTERNAL_TABLE") {
       tryCatch(
-        s3_path <- split_s3_uri(glue$get_table(DatabaseName = dbms.name,
-                                               Name = Table)$Table$StorageDescriptor$Location),
+        s3_path <- split_s3_uri(
+          conn@ptr$glue$get_table(DatabaseName = ll[["dbms.name"]], Name = ll[["table"]])[["Table"]][["StorageDescriptor"]][["Location"]]),
         error = function(e) py_error(e))
-      
       # Detect if key ends with "/" or if it has ".": https://github.com/DyfanJones/noctua/issues/125
       if(!grepl("\\.|/$", s3_path$key))
         s3_path$key <- sprintf("%s/", s3_path$key)
-      
+      all_keys <- list()
+      # Get all s3 objects linked to table
+      kwargs <- list(Bucket=s3_path$bucket, Prefix=s3_path$key)
+      while(is.null(kwargs$ContinuationToken)) {
+        objects <- do.call(conn@ptr$S3$list_objects_v2, kwargs)
+        all_keys <- c(all_keys, lapply(objects$Contents, function(x) list(Key=x$Key)))
+        if(identical(objects$NextContinuationToken, kwargs$ContinuationToken) || length(objects$NextContinuationToken) == 0) break
+        kwargs[["ContinuationToken"]] <- objects$NextContinuationToken
+      }
       message(paste0("Info: The S3 objects in prefix will be deleted:\n",
                      paste0("s3://", s3_path$bucket, "/", s3_path$key)))
-      
       if (!confirm) {
         confirm <- readline(prompt = "Delete files (y/n)?: ")
         if (tolower(confirm) != "y") {
           message("Info: Table deletion aborted.")
           return(NULL)}
       }
-      
-      # Remove objects in prefix of AWS Athena table
-      tryCatch(s3$Bucket(s3_path$bucket)$objects$filter(Prefix = s3_path$key)$delete(),
-               error = function(e) py_error(e))
+
+      # Only remove if files are found
+      if(length(all_keys) > 0){
+        # Delete S3 files in batch size 1000
+        key_parts <- split_vec(all_keys, 1000)
+        for(i in seq_along(key_parts))
+          conn@ptr$S3$delete_objects(Bucket = s3_path$bucket, Delete = list(Objects = key_parts[[i]]))
+      } else {
+        warning(sprintf('Failed to remove AWS S3 files from: "s3://%s/%s/". Please check if AWS S3 files exist.',
+                        s3_path$bucket, s3_path$key), call. = F)
+      }
     }
     
     # use glue to remove table from glue catalog
-    tryCatch(glue$delete_table(DatabaseName = dbms.name, Name = Table),
+    tryCatch(conn@ptr$glue$delete_table(DatabaseName = ll[["dbms.name"]], Name = ll[["table"]]),
              error = function(e) py_error(e))
     
     if (!delete_data) message("Info: Only Athena table has been removed.")
-    on_connection_updated(conn, Table)
+    on_connection_updated(conn, ll[["table"]])
     invisible(TRUE)
   })
 
@@ -673,7 +673,7 @@ setMethod(
   function(conn,
            statement = NULL, 
            statistics = FALSE, ...){
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    con_error_msg(conn, msg = "Connection already closed.")
     stopifnot(is.logical(statistics))
     rs <- dbSendQuery(conn, statement = statement)
     on.exit(dbClearResult(rs))
@@ -720,13 +720,12 @@ NULL
 setMethod(
   "dbGetInfo", "AthenaConnection",
   function(dbObj, ...) {
-    if (!dbIsValid(dbObj)) {stop("Connection already closed.", call. = FALSE)}
-    info <- dbObj@info
-    RegionName <- dbObj@ptr$region_name
+    con_error_msg(dbObj, msg = "Connection already closed.")
+    info <- as.list(dbObj@info)
     Boto <- as.character(boto_verison())
     rathena <- as.character(packageVersion("RAthena"))
-    info <- c(info, region_name = RegionName, boto3 = Boto, RAthena = rathena)
-    info
+    info <- c(info, boto3 = Boto, RAthena = rathena)
+    return(info)
   })
 
 #' Athena table partitions
@@ -775,15 +774,10 @@ setGeneric("dbGetPartition",
 setMethod(
   "dbGetPartition", "AthenaConnection",
   function(conn, name, ..., .format = FALSE) {
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    con_error_msg(conn, msg = "Connection already closed.")
     stopifnot(is.logical(.format))
-    
-    if(grepl("\\.", name)){
-      dbms.name <- gsub("\\..*", "" , name)
-      Table <- gsub(".*\\.", "" , name)
-    } else {dbms.name <- conn@info$dbms.name
-    Table <- name}
-    dt = dbGetQuery(conn, paste0("SHOW PARTITIONS ", dbms.name,".",Table))
+    ll <- db_detect(conn, name)
+    dt = dbGetQuery(conn, paste0("SHOW PARTITIONS ", ll[["dbms.name"]],".",ll[["table"]]))
     
     if(.format){
       # ensure returning format is data.table
@@ -835,23 +829,20 @@ NULL
 
 #' @rdname dbShow
 #' @export
-setGeneric("dbShow",
-           def = function(conn, name, ...) standardGeneric("dbShow"),
-           valueClass = "character")
+setGeneric(
+  "dbShow",
+  def = function(conn, name, ...) standardGeneric("dbShow"),
+  valueClass = "character"
+)
 
 #' @rdname dbShow
 #' @export
 setMethod(
   "dbShow", "AthenaConnection",
   function(conn, name, ...) {
-    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
-    
-    if(grepl("\\.", name)){
-      dbms.name <- gsub("\\..*", "" , name)
-      Table <- gsub(".*\\.", "" , name)
-    } else {dbms.name <- conn@info$dbms.name
-    Table <- name}
-    SQL(paste0(dbGetQuery(conn, paste0("SHOW CREATE TABLE ", dbms.name,".",Table))[[1]], collapse = "\n"))
+    con_error_msg(conn, msg = "Connection already closed.")
+    ll <- db_detect(conn, name)
+    SQL(paste0(dbGetQuery(conn, paste0("SHOW CREATE TABLE ", ll[["dbms.name"]],".",ll[["table"]]))[[1]], collapse = "\n"))
   })
 
 #' Simple wrapper to convert Athena backend file types
@@ -910,8 +901,10 @@ NULL
 
 #' @rdname dbConvertTable
 #' @export
-setGeneric("dbConvertTable",
-           def = function(conn, obj, name, ...) standardGeneric("dbConvertTable"))
+setGeneric(
+  "dbConvertTable",
+  def = function(conn, obj, name, ...) standardGeneric("dbConvertTable")
+)
 
 #' @rdname dbConvertTable
 #' @export
@@ -926,7 +919,7 @@ setMethod(
             compress = TRUE,
             data = TRUE,
             ...){
-     if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+     con_error_msg(conn, msg = "Connection already closed.")
      stopifnot(is.character(obj),
                is.character(name),
                is.null(partition) || is.character(partition),
