@@ -6,10 +6,10 @@ AthenaDataType <-
       integer =   "INT",
       integer64 = "BIGINT",
       numeric =   "DOUBLE",
-      double = "DOUBLE",
+      double =    "DOUBLE",
       factor =    "STRING",
       character = "STRING",
-      list = "STRING",
+      list =      "STRING",
       Date =      "DATE",
       POSIXct =   "TIMESTAMP",
       stop("Unknown class ", paste(class(fields), collapse = "/"), call. = FALSE)
@@ -22,8 +22,6 @@ AthenaToRDataType <- function(method, data_type) UseMethod("AthenaToRDataType")
 
 AthenaToRDataType.athena_data.table <- 
   function(method, data_type){
-    Names <- sapply(data_type, function(x) x$Name)
-    Types <- tolower(sapply(data_type, function(x) x$Type))
     athena_to_r <- function(x){
       switch(x,
              boolean = "logical",
@@ -33,21 +31,26 @@ AthenaToRDataType.athena_data.table <-
              smallint = "integer",
              bigint = athena_option_env$bigint,
              float = "double",
+             real = "double",
              decimal = "double",
              string = "character",
              varchar = "character",
+             char = "character",
              date = "Date",
              timestamp = "POSIXct",
+             array = "character",
+             row = "character",
+             map = "character",
+             json = "character",
+             ipaddress = "character",
+             varbinary = "character",
              x)}
-    output <- vapply(Types, athena_to_r, FUN.VALUE = character(1))
-    names(output) <- Names
+    output <- vapply(data_type, athena_to_r, FUN.VALUE = character(1))
     output
   }
 
 AthenaToRDataType.athena_vroom <- 
   function(method, data_type){
-    Names <- sapply(data_type, function(x) x$Name)
-    Types <- tolower(sapply(data_type, function(x) x$Type))
     athena_to_r <- function(x){
       switch(x,
              boolean = "l",
@@ -58,13 +61,20 @@ AthenaToRDataType.athena_vroom <-
              bigint = athena_option_env$bigint,
              double = "d",
              float = "d",
+             real = "d",
              decimal = "d",
              string = "c",
+             char = "c",
              varchar = "c",
              date = "D",
              timestamp = "T",
+             array = "c",
+             row = "c",
+             map = "c",
+             json = "c",
+             ipaddress = "c",
+             varbinary = "c",
              x)}
-    output <- vapply(Types, athena_to_r, FUN.VALUE = character(1))
-    names(output) <- Names
+    output <- vapply(data_type, athena_to_r, FUN.VALUE = character(1))
     output
   }
