@@ -97,7 +97,15 @@ setMethod(
 #' @param bigint The R type that 64-bit integer types should be mapped to,
 #'   default is [bit64::integer64], which allows the full range of 64 bit
 #'   integers.
+#' @param binary The R type that [binary/varbinary] types should be mapped to,
+#'   default is [raw]. If the mapping fails R will resort to [character] type.
+#'   To ignore data type conversion set to ["character"].
+#' @param json Attempt to converts AWS Athena data types [arrays, json] using \code{jsonlite:parse_json}. If the mapping fails R will resort to [character] type.
+#'   Custom Json parsers can be provide by using a function with data frame parameter.
+#'   To ignore data type conversion set to ["character"].
 #' @param keyboard_interrupt Stops AWS Athena process when R gets a keyboard interrupt, currently defaults to \code{TRUE}
+#' @param rstudio_conn_tab Optional to get AWS Athena Schema and display it in RStudio's Connections Tab.
+#'   Default set to \code{TRUE}.
 #' @param ... Any other parameter for Boto3 session: \href{https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html}{Boto3 session documentation}
 #' @aliases dbConnect
 #' @return \code{dbConnect()} returns a s4 class. This object is used to communicate with AWS Athena.
@@ -148,7 +156,10 @@ setMethod(
            region_name = NULL,
            botocore_session = NULL, 
            bigint = c("integer64", "integer", "numeric", "character"),
+           binary = c("raw", "character"),
+           json = c("auto", "character"),
            keyboard_interrupt = TRUE,
+           rstudio_conn_tab = TRUE,
            ...) {
     if(!py_module_available("boto3")){
       stop("Boto3 is not detected please install boto3 using either: `pip install boto3 numpy` in terminal or `install_boto()`.",
@@ -169,9 +180,14 @@ setMethod(
               is.null(role_arn) || is.character(role_arn),
               is.character(role_session_name),
               is.numeric(duration_seconds),
-              is.logical(keyboard_interrupt))
+              is.logical(keyboard_interrupt),
+              is.character(json) || is.function(json),
+              is.logical(rstudio_conn_tab))
     
     athena_option_env$bigint <- big_int(match.arg(bigint))
+    athena_option_env$binary <- match.arg(binary)
+    athena_option_env$json <- if(is.character(json)) jsonlite_check(json[[1]]) else json
+    athena_option_env$rstudio_conn_tab <- rstudio_conn_tab
     
     encryption_option <- switch(encryption_option[1],
                                 "NULL" = NULL,
